#include "mbed.h"
#include "w5uxhMbedKeyer.h"
#include "queues.h"

extern unsigned char kbdWPM;
extern unsigned char pdlWPM;
extern unsigned char weight;

extern Serial pc;
extern int cwBufSize;
extern Fifo *cwBuf;

extern char idString[10];

extern char focus;

char displayBuf[DISPLAY_BUF_SIZE];
int bufIndex = 0;


//                        ROW  COL
//                        ---  ---
#define PDLcursorSerial     1,   1
#define KBDcursorSerial     1,  11
#define WTcursorSerial      2,  19
#define TIMERcursorSerial   2,   1
#define BUFfreeCursorSerial 1,  26
#define BUFusedCursorSerial 2,  26
#define IDstringCursorSerial FIRST_DISPLAY_ROW-1,  1



/** -------------------------------------------------------------------------
@verbatim
Set the console mode to white on red background.

@endverbatim
**/
void setWhiteOnRed(void)
{
  pc.printf("\033[0;1;37;41m");
}

/** -------------------------------------------------------------------------
@verbatim
Set the console mode to white on green background.

@endverbatim
**/
void setWhiteOnGreen(void)
{
  pc.printf("\033[0;1;37;42m");
}



/** -------------------------------------------------------------------------
@verbatim
Set the console mode to white on blue background.

@endverbatim
**/
void setWhiteOnBlue(void)
{
  pc.printf("\033[0;1;37;44m");
}


/** -------------------------------------------------------------------------
@verbatim
Set the console mode to white on cyan background.

@endverbatim
**/
void setWhiteOnCyan(void)
{
  pc.printf("\033[0;1;37;46m");
}

/** -------------------------------------------------------------------------
@verbatim
Set the console mode to white on magneta background.

@endverbatim
**/
void setWhiteOnMagenta(void)
{
  pc.printf("\033[0;1;37;45m");
}

/** -------------------------------------------------------------------------
@verbatim
Set the console mode to white on yellow background.

@endverbatim
**/
void setWhiteOnYellow(void)
{
  pc.printf("\033[0;1;37;43m");
}

/** -------------------------------------------------------------------------
@verbatim
Set the color scheme back to default black on whie.

@endverbatim
**/
void setNormal(void)
{
  pc.printf("\033[0m");
}

/** -------------------------------------------------------------------------
@verbatim
Store the current cursor position.

@endverbatim
**/
void storeCursor(void)
{
  pc.printf("\033[s");
}

/** -------------------------------------------------------------------------
@verbatim
Restore the previously stored cursor position.

@endverbatim
**/
void restoreCursor(void)
{
  pc.printf("\033[u");
}


extern unsigned char toggleSameSpeedModeState;

/** -------------------------------------------------------------------------
@verbatim
Display speeds and weight on the console.

@endverbatim
**/
void displaySpeedsSerial(void)
{
  NVIC_Disable_the_UART_in_use;
  storeCursor();
  setCursor(PDLcursorSerial);
  if (toggleSameSpeedModeState == FALSE)
    {
      switch(focus)
    {
    case PDL:
      {
        setWhiteOnBlue();
        break;
      }
    case KBD: case WEIGHT:
      {
        setWhiteOnYellow();
        break;
      }
    }
    }
  else
    {
      setWhiteOnGreen();
    }
  pc.printf("PDL:%2d", pdlWPM);

  setCursor(KBDcursorSerial);
  if (toggleSameSpeedModeState == FALSE)
    {
      switch(focus)
    {
    case KBD:
      {
        setWhiteOnBlue();
        break;
      }
    case PDL: case WEIGHT:
      {
        setWhiteOnYellow();
        break;
      }
    }
    }
  else
    {
      setWhiteOnGreen();
    }
  
  pc.printf("KBD:%3d", kbdWPM);


  setCursor(WTcursorSerial);

  switch(focus)
    {
    case WEIGHT:
      {
    setWhiteOnBlue();
    break;
      }
    case PDL: case KBD:
      {
    setWhiteOnYellow();
    break;
      }
    }

  pc.printf("WT:%2d%%", weight);
  setNormal();
  restoreCursor();
  NVIC_Enable_the_UART_in_use;
}


/** -------------------------------------------------------------------------
@verbatim
Enable line wrap (not sure if this does anything useful).  I tried this in case it
allows backspace to continue back into the previous line.  It did not make any difference.

@endverbatim
**/
void enableSerialLineWrap(void)
{
  pc.printf("\033[7h");

}




/** -------------------------------------------------------------------------
@verbatim
Set a sub "window" to be scrolled, so we can leave a section at the top for
displaying settings.

@endverbatim
**/
void setConsoleScrollRows(int firstRow, int lastRow)
{
  pc.printf("\033[%d;%dr", firstRow, lastRow);

}



/** -------------------------------------------------------------------------
@verbatim
Clear the console screen.

@endverbatim
**/
void clearSerialScreen(void)
{
  pc.printf("\033[2J");
}


/** -------------------------------------------------------------------------
@verbatim
Set cursor position on console ANSI terminal

@endverbatim
**/
void setCursor(int row, int col)
{
  pc.printf("\x1b[%d;%dH", row, col);
}




/** -------------------------------------------------------------------------
@verbatim
Write a string at the specified cursor position

@endverbatim
**/
void writeStringAt(int row, int col, char *textString)
{
  setCursor (row, col);
  pc.printf("%s", textString);
}


/** -------------------------------------------------------------------------
@verbatim


@endverbatim
**/
void writeString(char *string)
{
  pc.printf(string, strlen (string));
}



/** -------------------------------------------------------------------------
@verbatim
Erase the current line.

@endverbatim
**/
void eraseLine(void)
{
  pc.printf("\033[2K");
}

static int previousUsedCount=4096;

/** -------------------------------------------------------------------------
@verbatim
Display the count of bytes remaining free in the cw buffer and the count
of the number of bytes in the buffer

@endverbatim
**/
void dsplyBufStatusSerial(int flag)
{
  int free;
  int used;

  if (flag == FORCE) previousUsedCount = 4096;

  used = cwBuf->get_count();
  if (used != previousUsedCount)
    {
      free = cwBufSize - used;

      NVIC_Disable_the_UART_in_use;
      storeCursor();
      setCursor(BUFfreeCursorSerial);
      pc.printf("Buf Free:  %5d", free);
      setCursor(BUFusedCursorSerial);
      pc.printf("Buf Used:  %5d", used);

      setNormal();
      restoreCursor();
      NVIC_Enable_the_UART_in_use;
    }
  previousUsedCount = used;

}




/** -------------------------------------------------------------------------
@verbatim
Display the current ten minute timer state, with visual warnings 
from 9 through 12 minutes.

@endverbatim

@param [in] minutes  Minutes count for ID timer
@param [in] seconds  Seconds count for ID timer
@param [in] timeTicIntrCount  0 = 1st half second interval 1 = 2nd half

**/

void displayTimerSerial(int minutes, int seconds, int timeTicIntrCount)
{
  NVIC_Disable_the_UART_in_use;
  storeCursor();
  setCursor(TIMERcursorSerial);

  if ((minutes < 9) || (minutes >12))
    {
      pc.printf("MIN:%02d sec:%02d", minutes, seconds);
    }
    else
      {
    setWhiteOnRed();
    pc.printf("MIN:%02d sec:%02d", minutes, seconds);
    setNormal();
      }
  restoreCursor();
  NVIC_Enable_the_UART_in_use;
}

void displayConsoleID(void)
{
  NVIC_Disable_the_UART_in_use;
  storeCursor();
  setCursor(IDstringCursorSerial);
  setWhiteOnGreen();
  pc.printf("                  %s                  ", idString);
  setNormal();
  restoreCursor();
  NVIC_Enable_the_UART_in_use;
}


#if 0
0 Normal text  30 Black foreground    40 Black background
1 Bold text    31 Red foreground      41 Red background
4 Underline    32 Green foreground    42 Green background 
5 Blink        33 Yellow foreground   43 Yellow background
7 Inverse      34 Blue foreground     44 Blue background
               35 Magenta foreground  45 Magenta background
               36 Cyan foreground     46 Cyan background
               37 White foreground    47 White background
#endif




