// We determine which serial port is in use in this section:
// The serial port over USB uses UART0.  This is the "special" USB to serial interface that works
// over the usb connector on the mbed.
// The RS-232 serial port that I use primarily uses UART1.
// m_mainUSBHostShell.cpp contains the Serial pc(xxx, yyy) declaration


//--------------------------------------------------------------------------------
// Only define one of the following UARTS in order to select the serial port to use
// #define USE_SERIAL_OVER_USB

#define USE_SERIAL_OVER_UART1
//--------------------------------------------------------------------------------

#ifdef USE_SERIAL_OVER_USB
#define NVIC_Disable_the_UART_in_use NVIC_DisableIRQ(UART0_IRQn)
#define NVIC_Enable_the_UART_in_use  NVIC_EnableIRQ(UART0_IRQn)
#endif


#ifdef USE_SERIAL_OVER_UART1
#define NVIC_Disable_the_UART_in_use    NVIC_DisableIRQ(UART1_IRQn)
#define NVIC_Enable_the_UART_in_use     NVIC_EnableIRQ(UART1_IRQn)
#endif

#ifdef USE_SERIAL_OVER_UART2
#define NVIC_Disable_the_UART_in_use    NVIC_DisableIRQ(UART2_IRQn)
#define NVIC_Enable_the_UART_in_use     NVIC_EnableIRQ(UART2_IRQn)
#endif

#ifdef USE_SERIAL_OVER_UART3
#define NVIC_Disable_the_UART_in_use    NVIC_DisableIRQ(UART3_IRQn)
#define NVIC_Enable_the_UART_in_use     NVIC_EnableIRQ(UART3_IRQn)
#endif

#define PC_BAUD 57600

#define KEY_UP   0
#define KEY_DOWN 1

#define TYPE_A 0
#define TYPE_B 1

#define KBD    0
#define PDL    1
#define WEIGHT 2

#define KBD_IDLE       0
#define KBD_DOT        1
#define KBD_DASH       2
#define KBD_EL_SPACE   3
#define KBD_CHAR_SPACE 4
#define KBD_WORD_SPACE 5

#define KBD_BUF_SIZE 1024
#define CMD_BUF_SIZE   10
#define IDLE 0
#define ACTIVE 1
#define TRUE 1
#define FALSE 0

#define FIRST_DISPLAY_ROW  8
#define LAST_DISPLAY_ROW  50
#define NR_DISPLAY_ROWS    (LAST_DISPLAY_ROW - FIRST_DISPLAY_ROW + 1)
#define NR_CHARS_PER_LINE 40
#define DISPLAY_BUF_SIZE (NR_DISPLAY_ROWS * NR_CHARS_PER_LINE)
#define FORCE 1
#define NO_FORCE 0

#define NO_TEXT     0
#define ROW2_ONLY   1
#define BOTH_ROWS   2

#define OFF 0
#define ON  1

#define AS 0x81
#define VE 0x82
#define AR 0x83
#define SK 0x84
#define BT 0x0D
#define AA 0x09

#define EOM_RESTORE_KBD_SPD 0x85  // End Of Msg:  for msgs preceeded by a QRS
#define SPEED_FOR_ANNOUNCE 30  // speed at which announcement msgs are sent via sidetone

#define CANNED_MSG_START 0x86
#define CANNED_MSG_END   0x87

void checkAutoFocus(int inputDevice);
void displayState(void);
void displayLcdStringRow(unsigned char row, char *string);
void atCwTimer(void);
void keyUp(void);
void keyDown(void);
void atCwTimerKbd(void);
void atCwTimerPdl(void);
void checkPaddles(void);
void getCompileTime(void);
void displayConsoleID(void);
void setFocus(char asciiChar);
void checkId(char newAsciiChar);
void sendCannedMsg(char asciiChar);
void previewMsg(char asciiChar);
int  fileOutputSettings(void);
int  fileInputSettings(void);
void dsplyBufStatusLCD(void);
void dsplyBufStatusSerial(int);
void checkCwBuffer(void);
void dsplyProsign(char asciiChar);
void checkKbdBuffer(void);
void setupKbdDash(void);
void setupKbdDot(void);
void setupKbdElementSpace(void);
void setupKbdCharSpace(void);
void writeKbdBuf(unsigned char data);
unsigned char asciiToMorseLookup (unsigned char ascii);
unsigned char morseToAsciiLookup (unsigned char morse);
void startCwCharacter(unsigned char morseByte);
void calcTimeDurationsKbd(unsigned char wpm, unsigned char weight);
void calcTimeDurationsPdl(unsigned char wpm, unsigned char weight);
void deltaSpeedKbd(int delta);
void deltaSpeedPdl(int delta);
void deltaWeight(int delta);
void resetCwBuffer(char asciiChar);
void displayTimerLCD(int minutes, int seconds, int timeTicIntrCount);
void displayTimerSerial(int minutes, int seconds, int timeTicIntrCount);
void displaySpeedsLCD(void);
void displaySpeedsSerial(void);
void backSpaceKbdBuf(void);
void setCursor(int row, int col);
void writeStringAt (int row, int col, char *textString);
void displayLcdTextRow(unsigned char row, unsigned char newChar);
void clearSerialScreen(void);
void clearLcdRow(int row);
void setConsoleScrollRows(int firstRow, int lastRow);
void writeString(char *string);
bool kbdBufDataAvailable(void);
void enableSerialLineWrap(void);
void keyerInit(void);
void keyerIdleLoop(void);
void initConsole(void);

